@@ -75,7 +75,7 @@ emptyMultilayer(
 RMLNetwork
 readMultilayer(
                const std::string& input_file,
-               const std::string& name, char sep, bool vertex_aligned);
+               const std::string& name, bool vertex_aligned);
 
 void
 writeMultilayer(
